package oldmcdata.evaluators.condor;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import oldmcdata.evaluators.AbstractCommandLineBuilder;
import oldmcdata.utils.StudyXMLHandler;

import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

/**
 * Title:
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.0
 */


public class CondorCommandBuilder extends AbstractCommandLineBuilder {
    private final static String TRANSFERINPUTFILES = "transfer_input_files";
    private Element condorElement, transferInputFiles, excursionTransferFileElement, studyTransferFileElement,
    condorArguments;
    private CondorSubmitFileCreator submitFileCreator;
    private File studyFile, submitDirectory, studyDirectory;

    public CondorCommandBuilder() {
        submitFileCreator = new CondorSubmitFileCreator();
    }

    public void buildCommandLines(File configFile, StudyXMLHandler studyXMLHandler,
                                  Map argumentMap, int numberOfExcursions) {
        this.configFile = configFile;
        this.studyXMLHandler = studyXMLHandler;
        this.argumentMap = argumentMap;
        this.numberOfExcursions = numberOfExcursions;

        try {
            openStudyXML();
            openConfigFile();
            setModelElement();
            setParameters();
            setReplicationsInfo();
            setPlaybacksWanted();
            createSubmitDirectory();
            buildCommandLines();
        } catch (Exception e) {
            e.printStackTrace(System.out);
            System.exit(-1);
        }
    }


    protected void setParameters() {
        studyDirectory = studyXMLHandler.getStudyDirectory();
        executableElement = getExecutableElement(CONDOR);
        if (executableElement == null)
            throw new RuntimeException("OLDMCDATA ERROR: can not find matching executable in " +
                    configFile.toString());
        executable = executableElement.attributeValue(VALUE);
        if (executable == null)
            throw new RuntimeException("OLDMCDATA ERROR: no executable specified in " +
                    configFile.toString());
        argumentsElement = executableElement.element(ARGUMENTS);
        // if arguments are not specified in the config file, then just use defaults of
        // false for all of the parameters
        if (argumentsElement == null)
            throw new RuntimeException("OLDMCDATA ERROR: an ordering of arguments in " +
                    configFile.toString() +
                    " has not been specified");
        Element modelTransferInputFiles = executableElement.element(TRANSFERINPUTFILES);
        condorElement = configRootElement.element(CONDOR);
        if (condorElement == null)
            throw new RuntimeException("OLDMCDATA ERROR: condor specification missing in " +
                    configFile.toString());

        // sets data common to all excursions
        condorElement.element("executable").setText(executable);
        condorElement.element("log").setText(modelName);
        condorElement.element("error").setText(modelName);
        condorElement.element("output").setText(modelName);
        transferInputFiles = condorElement.element(TRANSFERINPUTFILES);
		
        //always transfer the study.xml file
        studyFile = studyXMLHandler.getStudyFile();
        Element studyFileElement = DocumentHelper.createElement("file");
        studyFileElement.setText(studyFile.toString());
        transferInputFiles.add(studyFileElement);
        
		// add files that are specific to the model
        for (Iterator mti = modelTransferInputFiles.elements().iterator(); mti.hasNext();) {
            Element file = (Element) mti.next();
            modelTransferInputFiles.remove(file);
            transferInputFiles.add(file);
        }

        // add in mapfiles from study.xml
        Element modelParameters = studyXMLHandler.getModelParametersElement();
        List mapFiles = modelParameters.elements("MapFile");
        for (Iterator maps = mapFiles.iterator(); maps.hasNext();) {
            Element file = (Element) maps.next();
            Element filecopy = file.createCopy();
            filecopy.setText(getMapFileLocation(filecopy));
            transferInputFiles.add(filecopy);
        }

        String outputDir = new File(studyDirectory, "Output").toString();
        condorElement.element("initialdir").setText(outputDir);
    }

    /** Sets MapFileLocation to appropriate string. If MapFile is a file name,
     * assume MapFile is in study directory. This method will then append the study directory
     * to the MapFile name. If the MapFile includes a directory, then assume the
     * MapFile is in that absolute directory location.
     * @param file
     * @return
     */
    private String getMapFileLocation(Element file) {
        String mapFileName = file.getTextTrim();
        File mapFile = new File(mapFileName);
        String mapFileDir = mapFile.getParent();
        if (mapFileDir == null) { // then assume MapFile is in study directory
            mapFile = new File(studyDirectory, mapFileName);
        }
        if (!mapFile.exists())
            throw new RuntimeException("Can't locate MapFile: " + mapFile.toString());

        return mapFile.toString();
    }

    private void createSubmitDirectory() {
        submitDirectory = new File(studyDirectory, "submit");
        submitDirectory.mkdir();
    }

    protected void buildCommandLines() {
        studyFile = studyXMLHandler.getStudyFile();
        commandLines = new ArrayList();
        // the list of elements is now whatever is in the argumentElements
        // items could have been removed by the previous methods
        argumentList = argumentsElement.elements();
        condorArguments = condorElement.element(ARGUMENTS);
        condorArguments.setText("");

        for (int i = 0; i < numberOfExcursions; i++) {
            // construct an individual commandLine
            ArrayList commandLine = new ArrayList();
            addExcursionInfo();
            submitFileCreator.createSubmitFile(submitDirectory, condorElement, i);
            commandLine.add("condor_submit");
            commandLine.add(submitFileCreator.getSubmitFile());
            commandLines.add(commandLine);

            // clean up these elements for next excursion
            condorArguments.clearContent();
            transferInputFiles.remove(excursionTransferFileElement);
            transferInputFiles.remove(studyTransferFileElement);
        }

    }

    public void writeElement(Element el) {
        try {
            XMLWriter xmlWriter = new XMLWriter(System.out, OutputFormat.createPrettyPrint());
            xmlWriter.write(el);
            xmlWriter.close();
        } catch (IOException ioe) {
            System.out.println("Error attempting to write to System.out");
            ioe.printStackTrace(System.out);
            System.exit(-1);
        }
    }

    private void addExcursionInfo() {
        for (Iterator argIt = argumentList.iterator(); argIt.hasNext();) {
            Element argument = (Element) argIt.next();
            String argValue = argument.getText();
            Element condorArg = argument.createCopy();
            if (argValue.equals("NumberOfReplications")) {
                condorArg.setText(numberOfReplications);
                condorArguments.add(condorArg);
                continue;
            }
            if (argValue.equals("Seed")) {
                condorArg.setText((String) seedList.get(0));
                condorArguments.add(condorArg);
                continue;
            }
            if (argValue.equals("ExcursionFile")) {
                Iterator exfileit = (Iterator) argumentMap.get(argValue);
                File excursionFile = (File) exfileit.next();
                condorArg.setText(excursionFile.getName());
                condorArguments.add(condorArg);
                excursionTransferFileElement = transferInputFiles.addElement("file");
                excursionTransferFileElement.setText(excursionFile.toString());
                continue;
            }
            if (argValue.equals("MoeFile")) {
                Iterator moefileit = (Iterator) argumentMap.get(argValue);
                File moeFile = (File) moefileit.next();
                condorArg.setText(moeFile.getName());
                condorArguments.add(condorArg);
                continue;
            }
            if (argValue.equals("PlaybackFile")) {
                if (playbacksWanted) {
                    Iterator pbfileit = (Iterator) argumentMap.get(argValue);
                    File pbFile = (File) pbfileit.next();
                    condorArg.setText(pbFile.getName());
                    condorArguments.add(condorArg);
                }
                continue;
            }
            if (argValue.equals("Study")) {
                condorArg.setText(studyFile.getName());
                condorArguments.add(condorArg);
                studyTransferFileElement = transferInputFiles.addElement("file");
                studyTransferFileElement.setText(studyFile.toString());
                continue;
            }
            //otherwise, take argument literally
            condorArg.setText(argValue);
            condorArguments.add(condorArg);
            

        }

    }

}

