package oldmcdata.generators.utils;


import java.io.File;
import java.util.List;

import nalex.algorithms.Algorithm;
import nalex.problem.Problem;

import org.dom4j.DocumentHelper;
import org.dom4j.Element;


/** Converts the information in the XML setup file to data that the NALEX
 * software can use.
 * Title: AlgorithmConfigurationManager
 * Description:
 *Copyright: 2000 - 2005
 *  
 * @author Stephen C. Upton
 * @version 1.3
 */
public class AlgorithmConfigurationManager {
    protected final static String PROBLEMTYPE = "ProblemType",
    POPULATIONOUTPUT = "PopulationOutput",
    PARENT = "parent",
    OFFSPRING = "offspring",
    OPERATOR = "Operator",
    KEYNAME = "keyname",
    CLASSNAME = "classname",
    PARAMETER = "parameter",
    VALUE = "value",
    NAME = "name",
    POPULATION = "Population",
    TOURNAMENTSIZE = "TournamentSize",
    FITNESS = "Fitness",
    NUMBEROFEXCURSIONS = "NumberOfExcursions",
    NUMBEROFITERATIONS = "NumberOfIterations";
    private String problemType = "MAX", numberOfExcursions, numberOfIterations;
    private Element parameters, algorithmElement;
// variables associated with NALEX
    private Problem nalexProblem;
    private Algorithm algorithm;
    private PopulationWriter popWriter;

    public AlgorithmConfigurationManager() {
    }

    public Algorithm getAlgorithm() {
        return algorithm;
    }

    public void setProblemType(String noe) {
        problemType = noe;
    }

    public PopulationWriter getPopulationWriter() {
        return popWriter;
    }

    public void configure(File studyDirectory, Element parameters, List variables) {
        this.parameters = parameters;
        Element problemTypeElement = parameters.element(PROBLEMTYPE);
        Element popOutputFileElement = parameters.element(POPULATIONOUTPUT);
        // if any of these parameters are null, use defaults as specified in constructor
        if (problemTypeElement != null)
            setProblemType(problemTypeElement.getText());
        numberOfExcursions = parameters.element(NUMBEROFEXCURSIONS).getTextTrim();
        numberOfIterations = parameters.element(NUMBEROFITERATIONS).getTextTrim();
        popWriter = new PopulationWriter(studyDirectory);
        configurePopulationWriter(popOutputFileElement);
        configureAlgorithm();
        initializeProblem(variables);
        initializeAlgorithm();
    }

    private void configureAlgorithm() {
        algorithmElement = DocumentHelper.createElement("Algorithm");
        configurePopulation();
        configureSelectOperator();
        configureStopOperator();
        configureEvaluateOperator();
    }

    // the next set of methods are adapter methods, in that, they take input
    // information from the study.xml and convert it into information
    // useable by the NALEX algorithm setup process
    private void configurePopulation() {
        Element populationElement = DocumentHelper.createElement(POPULATION);
        populationElement.addAttribute(NAME, PARENT);
        populationElement.addAttribute("initialSize", numberOfExcursions);
        algorithmElement.add(populationElement);
    }

    private void configureSelectOperator() {
        Element ts = parameters.element(TOURNAMENTSIZE);
        if (ts == null) return;
        String tournamentSize = ts.getTextTrim();
        Element selectOperator = DocumentHelper.createElement(OPERATOR);
        selectOperator.addAttribute(KEYNAME, "select");
        Element parameter = DocumentHelper.createElement(PARAMETER);
        parameter.addAttribute(NAME, "NumberOfMatches");
        parameter.addAttribute(VALUE, tournamentSize);
        selectOperator.add(parameter);
        algorithmElement.add(selectOperator);
    }

    private void configureStopOperator() {
        Element stopOperator = DocumentHelper.createElement(OPERATOR);
        stopOperator.addAttribute(KEYNAME, "stop");
        Element parameter = DocumentHelper.createElement(PARAMETER);
        parameter.addAttribute(NAME, "MaxIterations");
        parameter.addAttribute(VALUE, numberOfIterations);
        stopOperator.add(parameter);
        algorithmElement.add(stopOperator);
    }

    private void configureEvaluateOperator() {
        Element evaluateOperator = DocumentHelper.createElement(OPERATOR);
        evaluateOperator.addAttribute(KEYNAME, "evaluate");
        evaluateOperator.addAttribute(CLASSNAME, "oldmcdata.generators.utils.NALEXModelEvaluate");
        algorithmElement.add(evaluateOperator);
    }


    private void configurePopulationWriter(Element popOutputFileElement) {
        boolean parentOutput = true, offspringOutput = true;
        if (popOutputFileElement != null) {
            String printParents = popOutputFileElement.attributeValue(PARENT);
            String printOffspring = popOutputFileElement.attributeValue(OFFSPRING);
            if (printParents != null)
                parentOutput = Boolean.getBoolean(printParents);
            if (printOffspring != null)
                offspringOutput = Boolean.getBoolean(printOffspring);
        }
        if (parentOutput)
            popWriter.addWriter("parents");
        if (offspringOutput)
            popWriter.addWriter("offspring");

    }


    private void initializeProblem(List variables) {
        nalexProblem = new NALEXModelProblem(problemType, variables);
    }

    /** EvolutionProgram uses 2 populations, a parent population
     *  and an offspring population with 5 Operators:
     * InitializeUniform - generates Individuals randomly within the bounds of the problem variables
     * MutateSelfAdaptLogNormal - adapts the variance of a Normal distribution used in generating offspring
     * NALEXModelEvaluate - uses fitness calculated by FitnessCalculator of the EP Generators Analyzer
     * TournamentSelection - calculates a ranking of Individuals based on a tournament played
     *  between each Individual and n randomly selected other Individuals in the population
     *  composed of both parents and offspring. The next parent population is then determined by
     *  the popSize number of highest ranking Individuals. This is an elitist selection operator
     * DefaultStop - stops after a specified number of generations
     */
    private void initializeAlgorithm() {
        try {
            algorithm = new algorithms.EvolutionProgram();
            algorithm.setBuilder("nalex.utils.builders.XMLAlgorithmBuilder");
            algorithm.setParameters(algorithmElement.elements());
            algorithm.setProblem(nalexProblem);
            algorithm.setUp();
        } catch (Exception anyex) {
            System.out.println("Failed to initialize algorithm for any reason");
            anyex.printStackTrace();
            System.exit(-1);
        }
    }
}






